@@ -33,6 +33,7 @@ import apt
 import changes
 import perform
 import util
+import debfile
 
 # When writing to a pipe where there is no reader (e.g., when
 # output is directed to head or to less and the user exists from less
@@ -50,33 +51,22 @@ def set_verbosity_level(new_level):
     verbose = new_level
 
 
-def commify(strnum):
-    strnum = str(strnum)
-    if len(strnum) <= 3:
-        return strnum
-    else:
-        pre = strnum[:-3]
-        post = strnum[-3:]
-        pre = commify(pre)
-        return pre + "," + post
-
-
 def ping_host(hostname):
     "Check if host is reachable."
 
     # Check if fping is installed.
     if perform.execute("fping localhost 2>/dev/null >/dev/null",
                        display=False) != 0:
-        print "JIG Warning: fping was not found. " +\
-              "Consider installing the package fping.\n"
+        print("JIG Warning: fping was not found. " +\
+              "Consider installing the package fping.\n")
 
     # Check if we can talk to the HOST
     elif perform.execute("fping " + hostname + " 2>/dev/null >/dev/null",
                          display=False) != 0:
-        print "JIG Warning: Could not contact the Debian server at " + hostname\
+        print("JIG Warning: Could not contact the Debian server at " + hostname\
         + """
              Perhaps it is down or you are not connected to the network.
-             JIG will continue to try to get the information required."""
+             JIG will continue to try to get the information required.""")
     else:
         return True  # host found
 
@@ -104,7 +94,7 @@ def get_available(command="dumpavail"):
     tmpcache = tempfile.mkstemp()[1]
     perform.execute("apt-cache dumpavail > " + tmpcache)
     perform.execute("cat /var/lib/dpkg/status >> " + tmpcache)  # 090501 fix
-    avail = apt_pkg.TagFile(file(tmpcache))
+    avail = apt_pkg.TagFile(open(tmpcache))
     if os.path.exists(tmpcache):
         os.remove(tmpcache)
     return avail
@@ -136,17 +126,17 @@ def do_dependents(package):
             if not con: con = ""
             if not rep: rep = ""
             if package in dep:
-                print "d", nam  # depends
+                print("d", nam)  # depends
             elif package in sug:
-                print "s", nam  # suggests
+                print("s", nam)  # suggests
             elif package in rec:
-                print "r", nam  # recommends
+                print("r", nam)  # recommends
             elif package in con:
-                print "c", nam  # conflicts
+                print("c", nam)  # conflicts
             elif package in rep:
-                print "p", nam  # replaces
+                print("p", nam)  # replaces
             else:
-                print "o", nam  # other
+                print("o", nam)  # other
 
 #------------------------------------------------------------------------
 #
@@ -163,7 +153,7 @@ def do_dependents(package):
 #
 #------------------------------------------------------------------------
 def do_describe(packages):
-    "Display a description of a package(s)."
+    """Display package description(s)."""
 
     #
     # From where do we get information about the packages?
@@ -205,7 +195,7 @@ def do_describe(packages):
     if package_files:
         for package_file in package_files:
             perform.execute("dpkg-deb --info " + package_file)
-            print "="*72
+            print("="*72)
             sys.stdout.flush()
 
     # 090409 Bug #432266 From Reuben Thomas <rrt@sc3d.org> 9 Apr 2009
@@ -249,20 +239,14 @@ def do_describe(packages):
 
     # Print out the one line descriptions. Should it be sorted?
     # If not sorted it should be same order as on the command line.
-    pkgs = describe_list.keys()
-    pkgs.sort()
-
-    # hack - ensures that we reach aptitude's 'virtual pkg' message
-    cache = apt.Cache()
-    if cache.get_providing_packages("motho"):
-        pkgs = [1]
+    pkgs = sorted(describe_list)
 
     # Print the description, depending on level of detail (verbose).
-    if len(pkgs) == 0 and verbose < 2:  # 'verbose < 2' is part of above hack
-        print "No packages found from those known to be available/installed."
+    if len(pkgs) == 0 and verbose < 2:  # 'verbose' is for handling virtual pkgs
+        print("No packages found from those known to be available/installed.")
     elif verbose == 0:
-        print "{0:24} {1}".format("Package", "Description")
-        print "="*24 + "-" + "="*51
+        print("{0:24} {1}".format("Package", "Description"))
+        print("="*24 + "-" + "="*51)
 
 #
 # We could start using the grep-dctrl package to get the information.
@@ -288,18 +272,18 @@ def do_describe(packages):
             if describe_list[pkg]:
                 line_end = describe_list[pkg].find("\n")
                 package_short_description = describe_list[pkg][0:line_end]
-            print "%-24s %s" % (pkg, package_short_description.capitalize())
+            print("%-24s %s" % (pkg, package_short_description.capitalize()))
     elif verbose == 1:
         for pkg in pkgs:
-            print pkg + ": " + describe_list[pkg].capitalize() + "\n"
+            print(pkg + ": " + describe_list[pkg].capitalize() + "\n")
     else:
         # TODO is there a way of doing this using apt_pkg easily?
         # Use "aptitude show" for now.
         #
 # Comment out for fix for Bug#366678
         package_names = util.concat(packages)
-        command = "aptitude show " + package_names
-        perform.execute(command)
+        cmd = "apt-cache" if util.fast else "aptitude"
+        perform.execute(cmd + " show " + package_names)
 # Bug#366678 fix from mvo - part 3 - not working yet???
 #   for pkgname in filter(lambda pkgname: cache.has_key(pkgname), packages):
 #       pkg = cache[pkgname]
@@ -324,11 +308,11 @@ def do_describe_new(install=False):
     #
     new_pkgs = changes.get_new_available()
     if len(new_pkgs) == 0:
-        print "No new packages"
+        print("No new packages")
     else:
         do_describe(new_pkgs)
         if install:
-            print "="*76
+            print("="*76)
             do_install(new_pkgs)
 
 #------------------------------------------------------------------------
@@ -369,9 +353,9 @@ def do_force(packages):
             elif os.path.exists(archives + pkg):
                 command += "'" + archives + pkg + "' "
             else:
-                print """Wajig: Error: File `%s' not found.
+                print("""Wajig: Error: File `%s' not found.
               Searched current directory and %s.
-              Please confirm the location and try again.""" % (pkg, archives)
+              Please confirm the location and try again.""" % (pkg, archives))
                 return()
     else:
         #
@@ -419,7 +403,7 @@ def do_hold(packages):
         # Do all of it as root then!
         command = "echo \"" + p + " hold\" | dpkg --set-selections"
         perform.execute(command, root=1)
-    print "The following packages are on hold:"
+    print("The following packages are on hold:")
     perform.execute("dpkg --get-selections | egrep 'hold$' | cut -f1")
 
 
@@ -428,7 +412,7 @@ def do_hold(packages):
 # INSTALL
 #
 #------------------------------------------------------------------------
-def do_install(packages, yes=False, noauth=False):
+def do_install(packages, yes="", noauth="", dist=""):
     "Install packages."
 
     #
@@ -451,7 +435,7 @@ def do_install(packages, yes=False, noauth=False):
         joined = str.join(stripped)
         packages = joined.split()
 
-    # rading packages from a file
+    # reading packages from a file
     elif len(packages) == 2 and packages[0] == "-f":
         stripped = [x.strip() for x in open(packages[1]).readlines()]
         joined = str.join(stripped)
@@ -461,8 +445,8 @@ def do_install(packages, yes=False, noauth=False):
     if re.match("(http|ftp)://", packages[0]) \
        and util.requires_package("wget", "/usr/bin/wget"):
         if len(packages) > 1:
-            print "wajig: Error: install URL allows only one URL, not " +\
-                  str(len(packages))
+            print("wajig: Error: install URL allows only one URL, not " +\
+                  str(len(packages)))
             sys.exit(1)
         tmpdeb = tempfile.mkstemp()[1] + ".deb"
         command = "wget --output-document=" + tmpdeb + " " + packages[0]
@@ -472,25 +456,12 @@ def do_install(packages, yes=False, noauth=False):
             if os.path.exists(tmpdeb):
                 os.remove(tmpdeb)
         else:
-            print "Wajig: The location " + packages[0] +\
-                  " was not found. Check and try again."
+            print("Wajig: The location " + packages[0] +\
+                  " was not found. Check and try again.")
 
-    # check if a specific .DEB file was specified
+    # check if DEB files were specified
     elif re.match(".*\.deb$", packages[0]):
-        command = "dpkg -i"
-        for pkg in packages:
-            if os.path.exists(pkg):
-                command = "{0} '{1}' ".format(command, pkg)
-            elif os.path.exists("/var/cache/apt/archives/" + pkg):
-                command = "{0} '/var/cache/apt/archives/{1}' ".format(command, pkg)
-            else:
-                print "Wajig: The file " + pkg +\
-                      " not found in either the" +\
-                      " current directory\n" +\
-                      "       nor in /var/cache/apt/archives/\n" +\
-                      "       Please confirm location and try again."
-                return
-        perform.execute(command, root=1)
+        debfile.install(set(packages))
     #
     # Check if a "/+" is in a package name then use the following distribution
     # for all packages! We might not want this previsely if there are multiple
@@ -511,15 +482,19 @@ def do_install(packages, yes=False, noauth=False):
 #                   (release, util.concat(packages))
 #       perform.execute(command, root=1)
     else:
-        perform.execute("apt-get {0} {1} install {2}".format(yes, noauth,
-                         util.concat(packages)), root=True)
+        rec = util.recommends()
+        if dist:
+            dist = "--target-release " + dist
+        cmd = "apt-get {0} {1} {2} {3} install {4}"
+        cmd = cmd.format(yes, noauth, rec, dist, util.concat(packages))
+        perform.execute(cmd, root=True)
 
 #------------------------------------------------------------------------
 #
 # INSTALL SUGGEST
 #
 #------------------------------------------------------------------------
-def do_install_suggest(packages, deptype, noauth=""):
+def do_install_suggest(packages, yes, noauth):
     "Install packages suggested by the listed packages."
 
     # If reading from standard input, generate the list of packages.
@@ -538,13 +513,8 @@ def do_install_suggest(packages, deptype, noauth=""):
     for section in avail:
         if section.get("Package") in packages:
             sug = section.get("Suggests")
-            rec = section.get("Recommends")
-            if deptype in ("Suggests", "Both"):
-                if sug:
-                    suggest_list = "{0} {1}".format(suggest_list, sug)
-            if deptype in ("Recommends", "Both"):
-                if rec:
-                    suggest_list = "{0} {1}".format(suggest_list, rec)
+            if sug:
+                suggest_list = "{0} {1}".format(suggest_list, sug)
 
     # Remove version information
     suggest_list = re.sub('\([^)]*\)', '', suggest_list)
@@ -562,15 +532,10 @@ def do_install_suggest(packages, deptype, noauth=""):
     for i in suggest_list.split():
         suggest_list = i + " " + re.sub(" " + i + " ", ' ', suggest_list)
 
-    # Use aptitude -r to get the recursive recommends installed, rather
-    # than going just one level - #402598.
-    if deptype == "Recommends":
-        command = "aptitude %s -r --show-deps --show-versions install " % \
-            noauth + util.concat(packages) + suggest_list
-    else:
-        command = "apt-get %s --show-upgraded install " % noauth \
-            + util.concat(packages) + suggest_list
-    perform.execute(command, root=1)
+    perform.execute("apt-get {0} {1} {2} --show-upgraded install {3} {4}".\
+                     format(util.recommends(), yes, noauth, \
+                     util.concat(packages), suggest_list),
+                     root=True)
 
 #-----------------------------------------------------------------------
 #
@@ -584,14 +549,14 @@ def do_listsections():
         s = section.get("Section")
         if s not in sections:
             sections.append(s)
-    print "\n".join(sections)
+    print("\n".join(sections))
 
 
 def do_listsection(pattern):
     avail = get_available()
     for section in avail:
         if (pattern == section.get("Section")):
-            print section.get("Package")
+            print(section.get("Package"))
 
 #------------------------------------------------------------------------
 #
@@ -637,7 +602,7 @@ def do_listscripts(pkg):
         for script in scripts:
             if "./" + script in "".join(pkgScripts):
                 nlen = (72 - len(script)) / 2
-                print ">"*nlen, script, "<"*nlen
+                print(">"*nlen, script, "<"*nlen)
                 command = "ar p " + pkg + " control.tar.gz |" +\
                           "tar zxvf - -O ./" + script +\
                           " 2>/dev/null"
@@ -648,7 +613,7 @@ def do_listscripts(pkg):
             fname = root + pkg + "." + script
             if os.path.exists(fname):
                 nlen = (72 - len(script))/2
-                print ">"*nlen, script, "<"*nlen
+                print(">"*nlen, script, "<"*nlen)
                 perform.execute("cat " + fname)
 
 #------------------------------------------------------------------------
@@ -659,16 +624,16 @@ def do_listscripts(pkg):
 def do_new():
     "Report on packages that are newly available."
 
-    print "%-24s %s" % ("Package", "Available")
-    print "="*24 + "-" + "="*16
+    print("%-24s %s" % ("Package", "Available"))
+    print("="*24 + "-" + "="*16)
     #
     # List each package and it's version
     #
     new_pkgs = changes.get_new_available()
     new_pkgs.sort()
     for i in range(0, len(new_pkgs)):
-        print "%-24s %s" % (new_pkgs[i],
-            changes.get_available_version(new_pkgs[i]))
+        print("%-24s %s" % (new_pkgs[i],
+            changes.get_available_version(new_pkgs[i])))
 
 
 #------------------------------------------------------------------------
@@ -686,31 +651,42 @@ def local_changelog(package, tmp):
     elif os.path.exists(changelog_native):
         return "zcat {0} >> {1}".format(changelog_native, tmp)
     else:
-        print "Package", package, "is likely broken (changelog not found)!"
+        print("Package", package, "is likely broken (changelog not found)!")
 
 
 def do_changelog(package, pager):
     """Display Debian changelog.
+
     network on:
          changelog - if there's newer entries, display them
       -v changelog - if there's newer entries, display them, and proceed to
                      display complete local changelog
-      -x changelog - same as "-v changelog", but use a pager
+      -P changelog - same as "-v changelog", but use a pager
+
     network off:
          changelog - if there's newer entries, mention failure to retrieve
       -v changelog - if there's newer entries, mention failure to retrieve, and
                      proceed to display complete local changelog
-      -x changelog - same as "-v changelog", but use a pager
+      -P changelog - same as "-v changelog", but use a pager
     """
 
-    changelog = "{0:=^72}\n".format(" {0} ".format(package))  # header
+    changelog = "{:=^79}\n".format(" {} ".format(package))  # header
     if pager:
         pipe_cmd = "/usr/bin/sensible-pager "
     else:
         pipe_cmd = ""
 
     pkg = apt.Cache()[package]
-    changelog += pkg.get_changelog()
+    try:
+        changelog += pkg.get_changelog()
+    except AttributeError as e:
+        # This is caught so as to avoid an ugly python-apt trace; it's a bug
+        # that surfaces when:
+        # 1. The package is not available in the default Debian suite
+        # 2. The suite the package belongs to is set to a pin of < 0
+        print("If this package is not on your default Debian suite, " \
+              "ensure that it's APT pinning isn't less than 0.")
+        return
     help_message = "\nTo display the local changelog, run:\n" \
                    "wajig --pager changelog " + package
     if "Failed to download the list of changes" in changelog:
@@ -723,13 +699,12 @@ def do_changelog(package, pager):
         if not verbose:
             changelog += help_message
     if not verbose:
-        print changelog
+        print(changelog)
     else:
         tmp = tempfile.mkstemp()[1]
         with open(tmp, "w") as f:
             if pkg.is_installed:
-                changelog += "\n{0:=^72}\n".format(" local changelog ")
-            changelog = changelog.encode("utf8")
+                changelog += "\n{:=^79}\n".format(" local changelog ")
             f.write(changelog)
         if pkg.is_installed:
             cmd = local_changelog(package, tmp)
@@ -757,38 +732,30 @@ def do_newupgrades(install=False):
     #
     new_upgrades = changes.get_new_upgrades()
     if len(new_upgrades) == 0:
-        print "No new upgrades"
+        print("No new upgrades")
     else:
-        print "%-24s %-24s %s" % ("Package", "Available", "Installed")
-        print "="*24 + "-" + "="*24 + "-" + "="*24
+        print("%-24s %-24s %s" % ("Package", "Available", "Installed"))
+        print("="*24 + "-" + "="*24 + "-" + "="*24)
         new_upgrades.sort()
         for i in range(0, len(new_upgrades)):
-            print "%-24s %-24s %-24s" % (new_upgrades[i], \
+            print("%-24s %-24s %-24s" % (new_upgrades[i], \
                             changes.get_available_version(new_upgrades[i]), \
-                            changes.get_installed_version(new_upgrades[i]))
+                            changes.get_installed_version(new_upgrades[i])))
         if install:
-            print "="*74
+            print("="*74)
             do_install(new_upgrades)
 
 
-########################################################################
-# SIZE
-#
 def do_size(packages, size=0):
-    "Print sizes for pkg in list PACAKAGES with size greater than SIZE."
-    #
+    "Print sizes for pkg in list PACKAGES with size greater than SIZE."
+
     # Work with the list of installed packages
     # (I think status has more than installed?)
-    #
     status = apt_pkg.TagFile(open("/var/lib/dpkg/status", "r"))
-    #
-    # Initialise the sizes dictionary.
-    #
-    size_list = {}
-    status_list = {}
-    #
+    size_list = dict()
+    status_list = dict()
+
     # Check for information in the Status list
-    #
     for section in status:
         if not packages or section.get("Package") in packages:
             package_name   = section.get("Package")
@@ -798,29 +765,20 @@ def do_size(packages, size=0):
                 if package_name not in size_list:
                     size_list[package_name] = package_size
                     status_list[package_name] = package_status
-    #
-    # Print out the one line descriptions.
-    #
-    pkgs = size_list.keys()
-    # pkgs.sort()
-    pkgs.sort(lambda x, y: cmp(int(size_list[x]), int(size_list[y])))
-    #
-    # Output
-    #
-    if len(pkgs) == 0:
-        print "No packages found from those known to be available or installed"
-    else:
-        print "%-30s %s       %s" % ("Package", "Size", "Status")
-        print "="*30 + "-" + "="*10 + "-" + "="*35
-        for pkg in pkgs:
-            print "%-30s %10s     %-20s" \
-                  % (pkg, commify(size_list[pkg]), status_list[pkg])
 
-#------------------------------------------------------------------------
-#
-# STATUS
-#
-#------------------------------------------------------------------------
+    pkgs = list(size_list)
+    pkgs.sort(key=lambda x: int(size_list[x]))  # sort by size
+
+    if len(pkgs) == 0:
+        print("No packages found from those known to be available or installed")
+    else:
+        print("{:<33} {:^10} {:>12}".format("Package", "Size (KB)", "Status"))
+        print("{}-{}-{}".format("="*33, "="*10, "="*12))
+        for pkg in pkgs:
+            print("{:<33} {:^10} {:>12}".format(pkg,
+                    format(int(size_list[pkg]), ',d'), status_list[pkg]))
+
+
 def do_status(packages, snapshot=False):
     """List status of the packages identified.
 
@@ -830,9 +788,9 @@ def do_status(packages, snapshot=False):
     """
 
     if not snapshot:
-        print "%-23s %-15s %-15s %-15s %s" % \
-              ("Package", "Installed", "Previous", "Now", "State")
-        print "="*23 + "-" + "="*15 + "-" + "="*15 + "-" + "="*15 + "-" + "="*5
+        print("%-23s %-15s %-15s %-15s %s" % \
+              ("Package", "Installed", "Previous", "Now", "State"))
+        print("="*23 + "-" + "="*15 + "-" + "="*15 + "-" + "="*15 + "-" + "="*5)
         sys.stdout.flush()
     #
     # Get status.  Previously used dpkg --list but this truncates package
@@ -886,7 +844,7 @@ def do_status(packages, snapshot=False):
     if snapshot:
         fobj = perform.execute(command, pipe=True)
         for l in fobj:
-            print "=".join(l.split()[0:2])
+            print("=".join(l.split()[0:2]))
     else:
         perform.execute(command, langC=True)
     #
@@ -895,9 +853,7 @@ def do_status(packages, snapshot=False):
     #
     for i in packages:
         if os.system("egrep '^" + i + " ' " + ifile + " >/dev/null"):
-            #
             # Package is not installed.
-            #
             command = \
               "join -a 2 " + previous_file + " " + available_file + " | " +\
               "awk 'NF==2 {print $1, \"N/A\", $2; next}{print}' | " +\
@@ -906,17 +862,12 @@ def do_status(packages, snapshot=False):
               " | awk '{printf(\"%-20s\\t%-15s\\t%-15s\\t%-15s\\n\", " +\
               "$1, \"N/A\", $2, $3)}'"
             perform.execute(command, langC=True)
-    #
+
     # Tidy up - remove the "installed file"
-    #
     if os.path.exists(ifile):
         os.remove(ifile)
 
-#------------------------------------------------------------------------
-#
-# TOUPGRADE
-#
-#------------------------------------------------------------------------
+
 def do_toupgrade():
     "List packages with Available version more recent than Installed."
 
@@ -927,18 +878,16 @@ def do_toupgrade():
     # you've installed a more recent version than in the distribution).
     # So now also add in a call to "dpkg --compare-versions" which slows
     # things down quite a bit!
-    #
-    print "%-24s %-24s %s" % ("Package", "Available", "Installed")
-    print "="*24 + "-" + "="*24 + "-" + "="*24
-    #
+    print("%-24s %-24s %s" % ("Package", "Available", "Installed"))
+    print("="*24 + "-" + "="*24 + "-" + "="*24)
+
     # List each upgraded pacakge and it's version.
-    #
     to_upgrade = changes.get_to_upgrade()
     to_upgrade.sort()
     for i in range(0, len(to_upgrade)):
-        print "%-24s %-24s %-24s" % (to_upgrade[i], \
+        print("%-24s %-24s %-24s" % (to_upgrade[i], \
                             changes.get_available_version(to_upgrade[i]), \
-                            changes.get_installed_version(to_upgrade[i]))
+                            changes.get_installed_version(to_upgrade[i])))
 
 #------------------------------------------------------------------------
 #
@@ -954,7 +903,7 @@ def do_unhold(packages):
         # Do all of it as root then.
         command = "echo \"" + package + " install\" | dpkg --set-selections"
         perform.execute(command, root=1)
-    print "The following packages are still on hold:"
+    print("The following packages are still on hold:")
     perform.execute("dpkg --get-selections | egrep 'hold$' | cut -f1")
 
 #------------------------------------------------------------------------
@@ -962,214 +911,19 @@ def do_unhold(packages):
 # UPDATE
 #
 #------------------------------------------------------------------------
-def do_update(query=0, quiet=False):
-    "Perform an update of the available packages."
-
-    if verbose > 0:
-        print "The Packages files listing the available packages is being \
-               updated."
-    if query > 0:  # ask user whether this should be done.
-        versionfile = open("/etc/debian_version")
-        debian_version = versionfile.read().split()[0]
-        if debian_version == "testing/unstable":
-            print """
-You appear to be running the `unstable' or `testing' distribution of
-Debian. It is very likely that your Packages files are out of date.
-Doing an UPDATE is recommended.
-"""
-        else:
-            print """
-You appear to be running the `stable' distribution of Debian. It is not
-likely that your Packages files are out of date. Doing an UPDATE is optional
-"""
-        doit = raw_input("Update? [Y/n] ")
-        if not re.match("^(y|Y|yes|Yes|YES)$", doit):
-            return
-    #
-    # Note that we do a "dselect update" rather than an
-    # "apt-get update" since the
-    # latter does not update dpkg's idea of what's available!
-    #
-    # if perform.execute("dselect update", root=1) == 0:
-    #
-    # 02/03/10 Go back to apt-get for a while.  Note that the available
-    # list is now obtained from apt cache, not dpkg lib, for the DESCRIBE
-    # command. I've also started using my own mirror and getting a syntax
-    # error when dpkg grabs apt's availables file.
-    #
-    # 02/03/26 Go back to dselect - The apt available file is not updated
-    # by apt-get so new was not finding the package descriptions.
-    #
-    if quiet:
-        #
-        # The quotes carefully chosen below are important. swapping "
-        # and ' when root is running and /bin/sh is used
-        # (rather than sudo) results in a syntax error due to too many
-        # ' being used.
-        #
-        quietopt = '| egrep -v "(http|ftp)"'
-    else:
-        quietopt = ""
-    if perform.execute("dselect update " + quietopt, root=1) == 0:
-    # if perform.execute("apt-get update", root=1) == 0:
-        #
-        # Only update the available list if the UPDATE succeeded
-        #
+def do_update():
+    if not perform.execute("apt-get update", root=1):
         changes.update_available()
-        #
-        # How many new upgrades are there?
-        #
-        print "There are " + changes.count_upgrades() + " new upgrades"
-    else:
-        print "Wajig: The update failed for some reason."
-        print "       Have you configured dselect lately?"
-        print "       It may need updating to use apt."
-        print "       Or perhaps you purposely interrupted me."
-
-        exit(1)
+        print("There are " + changes.count_upgrades() + " new upgrades")
 
 
-#------------------------------------------------------------------------
-#
-# WHICHPKG
-#
-#------------------------------------------------------------------------
-def do_whichpkg(filename):
-    """Search for a particular file or pattern in a Debian package.
-
-    Search the Debian package archive (local installed packages and
-    from the Debian package archive on packages.debian.org) for any
-    packages containing a file with FILENAME in its name. The results
-    are printed to standard output.
-
-    Argument:
-    filename    Pattern to find
-    """
-
-    packages_host = "packages.debian.org"
-    ping_host(packages_host)
-    #
-    # Print out a suitable heading
-    #
-    print "%-59s %-17s" % ("File Path", "Package")
-    print "="*59 + "-" + "="*17
-    print "INSTALLED"
-    sys.stdout.flush()
-
-    #
-    # Get the local information first.
-    #
-    # Could ensure that we don't repeat information but for now let's
-    # just get the information out!
-    #
-    # Also need to check for diversions (like when mutt and mutt-utf8 are
-    # both installed.
-    #
-    command = "dpkg -S " + filename + " 2>/dev/null " +\
-              "| sed 's|: *|:|' " +\
-              "| grep -v '^diversion by ' " +\
-              "| awk -F: '{printf(\"%-59s %-17s\\n\", $2, $1)}'"
-    perform.execute(command)
-    #
-    # Now obtain the information from the Debian server
-    #
-    print "-"*77
-    print "AVAILABLE"
-    results = tempfile.mkstemp()[1]
-    filename = filename.replace('+', '%2B')
-
-    # 100117 Bug#565666 Update to newer search syntax. TODO We should
-    # really also determine the version and arch and pop the correct
-    # ones in, rather than hard coding unstable and amd64.
-
-    # command = "wget --timeout=60 --output-document=" + results +\
-    #           " http://" + packages_host + "/cgi-bin" +\
-    #           "/search_contents.pl\?word=" + filename +\
-    #           "\&searchmode=searchfilesanddirs" +\
-    #           "\&case=insensitive\&version=unstable" +\
-    #           "\&arch=i386 2> /dev/null"
-
-    command = "wget --timeout=60 --output-document=" + results +\
-              " http://" + packages_host +\
-              "/search\?searchon=contents\&keywords=" + filename +\
-              "\&mode=filename" +\
-              "\&suite=unstable" +\
-              "\&arch=any 2> /dev/null"
-
-    perform.execute(command)
-    #
-    # Test for multiple page output and handle appropriately
-    #
-    command = "grep page= " + results + " 2>&1 > /dev/null"
-    if perform.execute(command) == 0:
-        #
-        # Multiple pages of output
-        #
-        # 071104 gjw - It appears we don't get multiple pages
-        # with the new HTML format, so maybe I don't need
-        # to fix this one up. Just do a cat so that if we do
-        # then someone is bound to notice!!!!
-        command = "cat " + results   # + " | " # +\
-                  # HTML FORMAT CHANGED Oct 2007
-                  # "egrep -v " +\
-                  # "'^ *<|^$|^Packages search page|^package |//W3C//' | " +\
-                  # "egrep -v 'page=' | " +\
-                  # "perl -p -e 's|<[^>]*>||g;s|<[^>]*$||g;s|^[^<]*>||g;'"
-        perform.execute(command)
-        #
-        # Loop over the other pages
-        #
-        command = "for i in $(grep page= " + results +\
-                  " | perl -p -e 's|.*page=||; s|&.*>||' | sort -u -k 1b,1" +\
-                  " | grep -v 1); " +\
-                  "do " +\
-                  "wget --timeout=60 --output-document=" + results +\
-                  " http://packages.debian.org/cgi-bin" +\
-                  "/search\?searchon=contents\&page=$i\&keywords=" + filename +\
-                  "\&version=unstable\&arch=amd64" +\
-                  "\&mode=path " +\
-                  " 2> /dev/null; " +\
-                  "cat " + results + " | " +\
-                  "egrep -v " +\
-                  "'^ *<|^$|^Packages search page|^package |//W3C//' | " +\
-                  "egrep -v 'page=' | " +\
-                  "perl -p -e 's|<[^>]*>||g;s|<[^>]*$||g;" +\
-                  "s|^[^<]*>||g;';" +\
-                  "done"
-        perform.execute(command)
-    else:
-        #
-        # A single page of output
-        #
-        command = "cat " + results + " | " +\
-                  "awk 'BEGIN { RS = \"</tr>\" } " +\
-                  "/\"keyword\"/ { print $0 }' | " +\
-                  "lynx -stdin -dump -nonumbers -nolist | " +\
-                  "egrep -v '^$' | " +\
-                  "sed 's|, |,|' | " +\
-                  "awk '{printf(\"%-59s %-17s\\n\", $1, $2\" \"$3\" \"$4)}'"
-                  # 071104 gjw HTML FORMAT CHANGED
-                  # "egrep -v " +\
-                  # "'^ *<|^$|^Packages search page|^package |//W3C//' | " +\
-                  # "perl -p -e 's|<[^>]*>||g;s|<[^>]*$||g;s|^[^<]*>||g;'"
-        perform.execute(command)
-    print "-"*77
-
-    if os.path.exists(results):
-        os.remove(results)
-
-#------------------------------------------------------------------------
-#
-# FINDPKG
-#
-#------------------------------------------------------------------------
 def do_findpkg(pkg):
     "Look for a particular pkg at apt-get.org."
 
     ping_host("www.apt-get.org")
 
     # Print out a suitable heading
-    print "Lines suitable for /etc/apt/sources.list\n"
+    print("Lines suitable for /etc/apt/sources.list\n")
     sys.stdout.flush()
 
     # Obtain the information from the Apt-Get server
@@ -1238,7 +992,7 @@ def do_recdownload(packages):
             packages.remove(package)
             continue
 
-    print "Calculating all dependencies..."
+    print("Calculating all dependencies...")
     for i in packages:
         tmp = get_deps_recursively(i, [])
         for i in tmp:
@@ -1248,12 +1002,12 @@ def do_recdownload(packages):
                 continue
             if i not in packageNames:
                 packageNames.append(i)
-    print "Packages to download to /var/cache/apt/archives:"
+    print("Packages to download to /var/cache/apt/archives:")
     for i in packageNames:
         # We do this because apt-get install dont list the packages to
         # reinstall if they don't need to be upgraded
-        print i,
-    print "\n"
+        print(i, end=' ')
+    print("\n")
 
     command = "apt-get --download-only --reinstall -u install " \
     + util.concat(packageNames)
@@ -1262,12 +1016,10 @@ def do_recdownload(packages):
 
 def versions(packages):
     if len(packages) == 0:
-        command = "apt-show-versions"
+        perform.execute("apt-show-versions")
     else:
-        command = ""
-    for package in packages:
-        command += "apt-show-versions " + package + "; "
-    perform.execute(command)
+        for package in packages:
+            perform.execute("apt-show-versions " + package)
 
 
 def rbuilddep(package):
