@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 #
 # WAJIG - Debian Command Line System Administrator
 #
@@ -25,6 +25,7 @@
 #
 import getopt
 import os
+import subprocess
 import sys
 import re
 import tempfile
@@ -132,9 +133,9 @@ def interactive_shell():
     prompt = "JIG> "
     while True:
         try:
-            cmdline = raw_input(prompt)
+            cmdline = input(prompt)
         except:
-            print ""
+            print("")
             return
         cmd = cmdline.split()
         if cmd:
@@ -170,12 +171,13 @@ def main():
         sys.argv += oldargv[i].split(",")
 
     try:
-        sopts = "bhnpqstvxy"
-        lopts = ["backup=", "help", "pause", "quiet", "simulate",
-                 "teaching", "verbose=", "version", "yes", "noauth", "pager"]
+        sopts = "bfhnPpqrRstvy"
+        lopts = ("backup=", "dist=", "fast", "help", "pause", "quiet",
+                 "recommends", "norecommends", "simulate", "teaching",
+                 "verbose=", "version", "yes", "noauth", "pager")
         opts, args = getopt.getopt(sys.argv[1:], sopts, lopts)
-    except getopt.error, e:
-        print e
+    except getopt.error as e:
+        print(e)
         documentation.usage()
         util.finishup(2)
 
@@ -188,17 +190,25 @@ def main():
             backup = True
         elif o == "--backup":
             if a in ("upgrade", "distupgrade") and len(sys.argv) < 4:
-                print 'Should be of the form "wajig --backup=BKDIR upgrade"'
+                print('Should be of the form "wajig --backup=BKDIR upgrade"')
                 util.finishup(1)
             backup = a
+        elif o == "--dist":
+            util.dist = a
+        elif o in ["-f", "--fast"]:
+            util.fast = True
         elif o in ["-p", "--pause"]:
             pause = True
             util.pause = True
-        elif o in ["-x", "--pager"]:
+        elif o in ["-P", "--pager"]:
             pager = True
             commands.set_verbosity_level(1)
         elif o in ["-q", "--quiet"]:
             perform.set_quiet()
+        elif o in ["-r", "--recommends"]:
+            util.recommends_flag = True
+        elif o in ["-R", "--norecommends"]:
+            util.recommends_flag = False
         elif o in ["-s", "--simulate"]:
             perform.set_simulate(True)
         elif o in ["-t", "--teaching"]:
@@ -221,7 +231,7 @@ def main():
             try:
                 verbose = int(a)
             except ValueError:
-                print 'Should be of the form "wajig --verbose=1 CMD"'
+                print('Should be of the form "wajig --verbose=1 CMD"')
                 util.finishup(1)
             commands.set_verbosity_level(verbose)
         elif o == "--version":
@@ -254,7 +264,7 @@ def main():
     # 081222 remove any commas - this makes it easier to copy and
     # paste from the security status email, for example.
 
-    args = filter(lambda x: x != "", args)
+    args = [x for x in args if x != ""]
 
     # Provide help up front - don't need to initialise the system to give help
 
@@ -310,6 +320,10 @@ def select_command(command, args, verbose):
             perform.execute("apt-get autoclean",
                              root=True)
 
+    elif command == "autoremove":
+        if util.requires_no_args(command, args):
+            perform.execute("apt-get autoremove", root=True)
+
     elif command in ["bug", "bugs", "reportbug"]:
         if util.requires_one_arg(command, args, "a single named package"):
             if util.requires_package("reportbug", "/usr/bin/reportbug"):
@@ -320,17 +334,18 @@ def select_command(command, args, verbose):
         if util.requires_args(command, args, "a list of package names") \
         and util.requires_package("sudo", "/usr/bin/sudo"):
             # First make sure dependencies are met
-            result = perform.execute("apt-get build-dep " +
-                                      util.concat(args[1:]),
+            result = perform.execute("apt-get {0} {1} build-dep {2}".format(\
+                                      yes, noauth, util.concat(args[1:])),
                                       root=True)
             if not result:
-                perform.execute("apt-get source -b " +
-                                 util.concat(args[1:]),
+                perform.execute("apt-get {0} source --build {1}".format(\
+                                 noauth, util.concat(args[1:])),
                                  root=True)
 
     elif command in ("builddepend", "builddep"):
         if util.requires_args(command, args, "a list of package names"):
-            perform.execute("apt-get build-dep " + util.concat(args[1:]),
+            perform.execute("apt-get {0} {1} build-dep {2}".format(yes, noauth,
+                             util.concat(args[1:])),
                              root=True)
 
     elif command in ("reverse-build-depends", "rbuilddeps"):
@@ -372,14 +387,12 @@ def select_command(command, args, verbose):
 
     elif command in ["detail", "details", "show"]:
         if util.requires_args(command, args, "a list of packages or package file"):
-            verbose = 2
-            commands.set_verbosity_level(verbose)
+            commands.set_verbosity_level(2)
             commands.do_describe(args[1:])
 
     elif command in ["detailnew", "newdetail"]:
         if util.requires_no_args(command, args):
-            verbose = 2
-            commands.set_verbosity_level(verbose)
+            commands.set_verbosity_level(2)
             commands.do_describe_new()
 
     elif command == "upgrade":
@@ -390,22 +403,26 @@ def select_command(command, args, verbose):
             and util.requires_package("fakeroot", "/usr/bin/fakeroot") \
             and util.requires_no_args(command, args):
                 changes.backup_before_upgrade(backup, pkgs)
-            perform.execute("apt-get {0} {1} -u upgrade".format(yes, noauth),
-                             root=True)
+            if sys.argv[-1].lower() == "upgrade":
+                perform.execute("apt-get {0} {1} --show-upgraded upgrade".format(yes, noauth),
+                                 root=True)
+            else:
+                print("To upgrade individual packages, use INSTALL command:")
+                print("$ wajig INSTALL " + sys.argv[-1].lower())
         else:
-            print 'No upgrades. Did you run "wajig update" beforehand?'
+            print('No upgrades. Did you run "wajig update" beforehand?')
 
     elif command == "distupgrade":
         pkgs = util.upgradable(distupgrade=True)
         if not pkgs and len(args) < 2:
-            print 'No upgrades. Did you run "wajig update" beforehand?'
+            print('No upgrades. Did you run "wajig update" beforehand?')
         elif util.requires_opt_arg(command, args,
                                   "the distribution to upgrade to"):
             if backup \
             and util.requires_package("dpkg-repack", "/usr/bin/dpkg-repack") \
             and util.requires_package("fakeroot", "/usr/bin/fakeroot"):
                 changes.backup_before_upgrade(backup, pkgs, distupgrade=True)
-            cmd = "apt-get -u {0} {1} ".format(yes, noauth)
+            cmd = "apt-get --show-upgraded {0} {1} ".format(yes, noauth)
             if len(args) == 2:
                 cmd += "-t " + args[1] + " "
             cmd += "dist-upgrade"
@@ -425,7 +442,7 @@ def select_command(command, args, verbose):
             #
             # Print message here since no messages are printed for the command.
             #
-            print "Packages being downloaded to /var/cache/apt/archives..."
+            print("Packages being downloaded to /var/cache/apt/archives...")
             #
             # Do the download, non-interactively (--quiet),
             # and force download for already installed packages (--reinstall)
@@ -482,7 +499,7 @@ def select_command(command, args, verbose):
 
     elif command == "fixconfigure":
         if util.requires_no_args(command, args):
-            perform.execute("dpkg --configure -a",
+            perform.execute("dpkg --configure --pending",
                              root=True)
 
     elif command == "fixinstall":
@@ -519,28 +536,20 @@ def select_command(command, args, verbose):
             # it's not even advertised no more (removed from docs)
             if command == "autoinstall":
                 yes = "--yes"
-            commands.do_install(args[1:], yes, noauth)
-
-    elif command in ["installr", "recommended"]:
-        if util.requires_args(command, args, "a list of packages"):
-            commands.do_install_suggest(args[1:], "Recommends")
-
-    elif command == "installrs":
-        if util.requires_args(command, args, "a list of packages"):
-            commands.do_install_suggest(args[1:], "Both")
+            commands.do_install(args[1:], yes, noauth, util.dist)
 
     elif command in ["installs", "suggested"]:
         if util.requires_args(command, args, "a list of packages"):
-            commands.do_install_suggest(args[1:], "Suggests")
+            commands.do_install_suggest(args[1:], yes, noauth)
 
-    elif re.compile(r'install.*').match(command):
+    elif args[0].startswith('install') and "/" in args[0]:
         # For example: install/unsable
-        if util.requires_args(command, args,
-                         "a list of packages, .deb files, or url"):
-            perform.execute("apt-get --target-release {0} install {1}".\
-                             format(re.compile(r'install').sub("", command),
-                             util.concat(args[1:])),
-                             root=True)
+        util.requires_args(args[0], args,
+                          "a list of packages, .deb files, or url")
+        dist = args[0].split("/")[1]
+        perform.execute("apt-get --target-release {0} install {1}".\
+                         format(dist, util.concat(args[1:])),
+                         root=True)
 
     elif command == "integrity":
         if util.requires_no_args(command, args):
@@ -641,7 +650,6 @@ def select_command(command, args, verbose):
         perform.execute("cat /var/log/apt/history.log")
 
     elif command == "listnames":
-        # pdb.set_trace()
         if util.requires_opt_arg(command, args, "at most one argument"):
             commands.do_listnames(args[1:])
 
@@ -691,8 +699,7 @@ def select_command(command, args, verbose):
             elif args[1].lower() == "install":
                 commands.do_describe_new(install=True)
             else:
-                print "WaJIG Error: NEW only accepts optional " +\
-                      "argument INSTALL"
+                print("WaJIG Error: NEW only accepts optional argument INSTALL")
                 util.finishup(1)
 
     elif command in ["newupgrades", "newupgrade"]:
@@ -702,8 +709,8 @@ def select_command(command, args, verbose):
             elif args[1].lower() == "install":
                 commands.do_newupgrades(install=True)
             else:
-                print "WaJIG Error: NEWUPGRADES only accepts " +\
-                      "optional argument INSTALL"
+                print("WaJIG Error: NEWUPGRADES only accepts " + \
+                      "optional argument INSTALL")
                 util.finishup(1)
 
     elif command == "nonfree":
@@ -716,7 +723,7 @@ def select_command(command, args, verbose):
             if util.requires_package("deborphan", "/usr/bin/deborphan"):
                 perform.execute("deborphan")
 
-    elif command == "policy":
+    elif command in ("policy", "available"):
         perform.execute("apt-cache policy " + util.concat(args[1:]))
 
     elif command in ("purge", "purgedepend"):
@@ -752,7 +759,7 @@ def select_command(command, args, verbose):
         if util.requires_one_arg(command, args, "a single package"):
             docpath = "/usr/share/doc/" + args[1] + "/"
             if not os.path.exists(docpath):
-                print "No docs found for '{0}'. Is it installed?".format(args[1])
+                print("No docs found for '{0}'. Is it installed?".format(args[1]))
                 return
             if command == "news":
                 li = ("NEWS.Debian", "NEWS")
@@ -767,11 +774,11 @@ def select_command(command, args, verbose):
                     cat = "zcat"
                 if os.path.exists(path):
                     found = True
-                    print "{0:=^72}".format(" {0} ".format(x))
+                    print("{0:=^72}".format(" {0} ".format(x)))
                     sys.stdout.flush()
                     perform.execute(cat + " " + path)
             if not found:
-                print "No {0} file found for {1}.".format(command.upper(), args[1])
+                print("No {0} file found for {1}.".format(command.upper(), args[1]))
 
     elif command in "listrecommended":
         command = "aptitude search '" + \
@@ -821,7 +828,7 @@ def select_command(command, args, verbose):
         if util.requires_one_arg(command, args, "name of an installed package") \
         and util.requires_package("dpkg-repack", "/usr/bin/dpkg-repack") \
         and util.requires_package("fakeroot", "/usr/bin/fakeroot"):
-            perform.execute("fakeroot -u dpkg-repack " + args[1],
+            perform.execute("fakeroot --unknown-is-real dpkg-repack " + args[1],
                              root=False)
 
     elif command == "rpminstall":
@@ -847,10 +854,10 @@ def select_command(command, args, verbose):
                                  util.concat(args[1:]))
 
     elif command == "searchapt":
-        if util.requires_one_arg(command, args, "one of stable|testing|unstable"):
-            util.requires_package("netselect-apt", "/usr/bin/netselect-apt")
-            perform.execute("netselect-apt " + args[1],
-                             root=True)
+        util.requires_one_arg(command, args, "one of stable|testing|unstable")
+        util.requires_package("netselect-apt", "/usr/bin/netselect-apt")
+        perform.execute("netselect-apt " + args[1],
+                         root=True)
 
     elif command == "showdistupgrade":
         if util.requires_no_args(command, args):
@@ -882,14 +889,9 @@ def select_command(command, args, verbose):
             commands.do_status([], snapshot=True)
 
     elif command == "source":
-        if util.requires_args(command, args, "a list of package names"):
-            # First make sure dependencies are met
-            # John V. Belmonte 04 Nov 2005 requested this not be done
-            # Leave it to the user to do wajig builddepend
-            # It is not always necessary
-            #perform.execute("apt-get build-dep " + util.concat(args[1:]),
-            #                root=True)
-            perform.execute("apt-get source " + util.concat(args[1:]))
+        util.requires_args(command, args, "a list of package names")
+        util.requires_package("dpkg-source", "/usr/bin/dpkg-source")
+        perform.execute("apt-get source " + util.concat(args[1:]))
 
     elif command == "status":
         commands.do_status(args[1:])
@@ -897,12 +899,11 @@ def select_command(command, args, verbose):
     elif command in ["statusmatch", "statussearch"]:
         if util.requires_one_arg(command, args,
         "a search string for the package name"):
-            pkgs = map(lambda s: s.strip(),
-                   commands.do_listnames(args[1:], pipe=True).readlines())
+            pkgs = [s.strip() for s in commands.do_listnames(args[1:], pipe=True).readlines()]
             if len(pkgs) > 0:
                 commands.do_status(pkgs)
             else:
-                print "No packages found matching '{0}'".format(args[1])
+                print("No packages found matching '{0}'".format(args[1]))
         #
         # Simplest thing to do is call wajig again.  Not the right way
         # but works for now.
@@ -914,10 +915,9 @@ def select_command(command, args, verbose):
         #                    + " | xargs wajig status ")
 
     elif command == "tasksel":
-        if util.requires_no_args(command, args):
-            if util.requires_package("tasksel", "/usr/bin/tasksel"):
-                perform.execute("tasksel",
-                                 root=True)
+        util.requires_no_args(command, args)
+        util.requires_package("tasksel", "/usr/bin/tasksel")
+        perform.execute("tasksel", root=True)
 
     elif command == "toupgrade":
         if util.requires_no_args(command, args):
@@ -931,8 +931,8 @@ def select_command(command, args, verbose):
         # TODO Perhaps I can use map to "execute" over each package
 
     elif command == "update":
-        if util.requires_no_args(command, args):
-            commands.do_update()
+        util.requires_no_args(command, args)
+        commands.do_update()
 
     # For testing only!
     elif command == "updateavailable":
@@ -951,10 +951,10 @@ def select_command(command, args, verbose):
                                  root=True)
 
     elif command == "updateusbids":
-        if util.requires_package("usbutils", "/usr/sbin/update-usbids"):
-            if util.requires_no_args(command, args):
-                perform.execute("update-usbids",
-                                 root=True)
+        if util.requires_package("usbutils", "/usr/sbin/update-usbids") \
+        and util.requires_no_args(command, args):
+            perform.execute("update-usbids",
+                             root=True)
 
     elif command == "upgradesecurity":
         sources_list = tempfile.mkstemp(".security", "wajig.", "/tmp")[1]
@@ -963,10 +963,10 @@ def select_command(command, args, verbose):
         sources_file.write("deb http://security.debian.org/ " +\
                            "testing/updates main contrib non-free\n")
         sources_file.close()
-        perform.execute("apt-get --no-list-cleanup -o Dir::Etc::SourceList=" +\
+        perform.execute("apt-get --no-list-cleanup --option Dir::Etc::SourceList=" +\
                         "{0} update".format(sources_list),
                         root=True)
-        perform.execute("apt-get -o Dir::Etc::SourceList={0} upgrade".\
+        perform.execute("apt-get --option Dir::Etc::SourceList={0} upgrade".\
                          format(sources_list),
                          root=True)
         if os.path.exists(sources_list):
@@ -985,19 +985,22 @@ def select_command(command, args, verbose):
             commands.versions(args[1:])
 
     elif command in ["whichpkg", "whichpackage"]:
-        if util.requires_one_arg(command, args, "a filename (possibly with a path)") \
-        and util.requires_package("wget", "/usr/bin/wget") \
-        and util.requires_package("lynx", "/usr/bin/lynx"):
-            commands.do_whichpkg(args[1])
+        util.requires_one_arg(command, args, "a filename (or a path)")
+        out = subprocess.getstatusoutput("dpkg --search " + args[1])
+        if out[0]:  # didn't find matching package, so use the slower apt-file
+            util.requires_package("apt-file", "/usr/bin/apt-file")
+            perform.execute("apt-file search " + args[1])
+        else:
+            print(out[1])
 
     else:
         if command == args[0]:
-            print "The command {0} was not recognised.".format(command.upper())
+            print("The command {0} was not recognised.".format(command.upper()))
         else:
-            print "The command {0} (entered as {1}) was not recognised.".\
-                   format(command.upper(), argd[0])
-        print "Perhaps it is not yet implemented or you misspelt it."
-        print "Try 'wajig help' for further information."
+            print("The command {0} (entered as {1}) was not recognised.".\
+                   format(command.upper(), args[0]))
+        print("Perhaps it is not yet implemented or you misspelt it.")
+        print("Try 'wajig help' for further information.")
 
 
 #------------------------------------------------------------------------
